# Algorithmes et structures de données

# Structure de file d'attente

*****

## 1. Définition

Une file d'attente ou queue est une structure de données dans laquelle la première information entrée est la première à ressortir (en anglais : FIFO, First In First Out)

On considère ici une implémentation de queue dynamique et donc à capacité « infinie » (dépend en réalité de la mémoire disponible).

## 2. Spécification de la structure

Si le  type des éléments qu'on insère, n'est pas précisé, on parle de spécification générique.

Les opérations possibles sur une file d'attente (ou queue) en définissent l'interface :

* Opérations de manipulation
    * Créer une liste vide
    * Détruire une liste
    * Insérer un nouvel élément en queue de la file d'attente
    * Extraire un élément en tête de la file d'attente (i.e. le premier élément introduit)
    * 
* Fonctions de consultation
    * Lire l'élément en tête de la file d'attente 
    * Lire l'élément en queue de la file d'attente 
    * Tester si la file d'attente est vide 

L'interface de la file d'attente est l'ensemble des fonctionnalités offertes à l'utilisateur pour interagir 
avec celle-ci.

## 3. Structure de file d'attente

On considère une liste chaînée d'articles avec deux pointeurs de tête et de queue. Chaque article comportera un champ pour stocker les valeurs dans cette liste et un champ contenant une variable de type accès pour assurer le chaînage. On considère à titre d'exemple le cas de valeurs entières.

Schématiquement:

![Illustration d'une file d'attente](./figures/fig_queue_representation.png)

L'accès à la file d'attente se fera par les pointeurs de tête et de queue.

Une telle file d'attente basée sur une liste chaînée sera déclarée par exemple sous la forme suivante:

```C
    struct _element {  // Elément de liste
       int data;
       struct _element* next;
    } element;

    struct _queue {    // File d'attente:
       element* tete;  //    tête de file d'attente
       element* debut; //    queue de file d'attente
    }
```

## 4. Implémentation des fonctionnalité d'une file d'attente

### a) Consultations

On considère la fonction queue_est_vide qui teste si la file d'attente est vide et les fonctions `queue_tete` et `queue_debut` qui retournent l'élément en tête, respectivement en queue, de la file d'attente.

```C
    bool queue_est_vide(queue fa) {
       return (NULL == fa.tete && NULL == fa.debut);
    }

    int queue_tete(queue fa) {
       assert(!queue_est_vide(fa));
       return fa.tete­>data;
    }

    int queue_debut(queue fa) {
       assert(!queue_est_vide(fa));
       return fa.debut­>data;
    }
```

### b) Manipulations

#### Insertion en queue de file d'attente

Voici l'entête de la fonction: `void queue_inserer(queue* fa,int val);`

Il faut considérer plusieurs cas:

* La file d'attente est vide

![Insertion dans une file d'attente vide](./figures/fig_empty_queue_insert.png)

* L'insertion se fait en queue d'une file d'attente non vide

![Insertion dans une file d'attente non-vide](./figures/fig_non_empty_queue_insert.png)

#### Extraction en tête de file d'attente

Voici l'entête de la procédure: `int queue_extraire(queue* fa);`

On commence par récupérer, la valeur en tête de file d'attente via l'appel `queue_tete(*fa);`

Puis on met un pointeur temporaire sur l'élément en tête, avant de déplacer le pointeur de tête sur  l'élément suivant.    
Finalement, on désalloue la mémoire

![Extraction d'une file d'attente](./figures/fig_queue_extract.png)

Si la file d'attente n'avait qu'un seul élément, alors il faudrait mettre le pointeur `fa-­>debut` à `NULL`.   
Dans ce cas, à la suite du point (3), le pointeur `fa-­>tete` se retrouve à `NULL`. On doit donc ajouter l'instruction :

```C
    if (NULL == fa­>tete) {
       fa­>debut = NULL;
    }
```
